@@ -55,6 +55,7 @@ import logging
 import itertools
 import os
 import random
+import heapq
 
 import numpy
 import scipy
@@ -149,12 +150,19 @@ class Similarity(interfaces.SimilarityABC):
         >>> index[query] # ... then result will have 7 floats
         [0.0, 0.0, 0.2, 0.13, 0.8, 0.0, 0.1]
 
-        If `num_best` is set, queries return only the `num_best` most similar documents:
+        If `num_best` is set, queries return only the `num_best` most similar documents,
+        always leaving out documents for which the similarity is 0
+        (i.e. vectors with all dimensions zero and vectors with non-zero dimensions, but none in common).
+        If the input vector itself has only dimensions with value zero (the sparse representation is empty),
+        the returned list will always be empty.
 
         >>> index.num_best = 3
         >>> index[query] # return at most "num_best" of `(index_of_document, similarity)` tuples
         [(4, 0.8), (2, 0.13), (3, 0.13)]
 
+        You can also override `num_best` dynamically, simply by setting e.g.
+        `self.num_best = 10` before doing a query.
+
         """
         if output_prefix is None:
             # undocumented feature: set output_prefix=None to create the server in temp
@@ -201,9 +209,9 @@ class Similarity(interfaces.SimilarityABC):
             else:
                 doclen = len(doc)
                 if doclen < 0.3 * self.num_features:
-                    doc = matutils.corpus2csc([doc], self.num_features).T
+                    doc = matutils.unitvec(matutils.corpus2csc([doc], self.num_features).T)
                 else:
-                    doc = matutils.sparse2full(doc, self.num_features)
+                    doc = matutils.unitvec(matutils.sparse2full(doc, self.num_features))
             self.fresh_docs.append(doc)
             self.fresh_nnz += doclen
             if len(self.fresh_docs) >= self.shardsize:
@@ -226,14 +234,10 @@ class Similarity(interfaces.SimilarityABC):
 
         **NOTE**: the shard is closed even if it is not full yet (its size is smaller
         than `self.shardsize`). If documents are added later via `add_documents()`,
-        this incomplete shard will be loaded again and completed. For this reason,
-        avoid the pattern of calling `add_documents` followed by a query, with only a few
-        documents added. The re-opening makes this pattern inefficient. Instead,
-        try to add as many documents as possible (ideally, all of them), only then query.
+        this incomplete shard will be loaded again and completed.
         """
         if not self.fresh_docs:
             return
-        self.fresh_docs = matutils.Scipy2Corpus(self.fresh_docs)
         shardid = len(self.shards)
         # consider the shard sparse if its density is < 30%
         issparse = 0.3 > 1.0 * self.fresh_nnz / (len(self.fresh_docs) * self.num_features)
@@ -275,25 +279,40 @@ class Similarity(interfaces.SimilarityABC):
         """
         self.close_shard() # no-op if no documents added to index since last query
 
-        results = []
+        # reset num_best and normalize parameters, in case they were changed dynamically
         for shard in self.shards:
             shard.num_best = self.num_best
             shard.normalize = self.normalize
-            results.append(shard[query])
+
+        # there are 4 distinct code paths, depending on whether input `query` is
+        # a corpus (or numpy/scipy matrix) or a single document, and whether the
+        # similarity result is a full array or only num_best most similar documents.
 
         if self.num_best is None:
-            return numpy.hstack(results)
+            # user asked for all documents => just stack the sub-results into a single matrix
+            # (works for both corpus / single doc query)
+            return numpy.hstack(shard[query] for shard in self.shards)
 
-        # only top-n most similars requested; merge the partial results from all shards
-        is_corpus, results = utils.is_corpus(results)
-        if is_corpus:
-            # query = single document?
-            result = sorted(sum(results, []), key=lambda item: -item[1])[ : self.num_best]
-        else:
-            result = []
-            for parts in itertools.izip(*results):
-                merged = sorted(sum(parts, []), key=lambda item: -item[1])[ : self.num_best]
-                result.append(merged)
+        offsets = numpy.cumsum([0] + [len(shard) for shard in self.shards])
+        convert = lambda doc, shard_no: [(doc_index + offsets[shard_no], sim)
+                                         for doc_index, sim in doc]
+        is_corpus, query = utils.is_corpus(query)
+        is_corpus = is_corpus or hasattr(query, 'ndim') and query.ndim > 1 and query.shape[0] > 1
+        if not is_corpus:
+            # user asked for num_best most similar and query is a single doc
+            results = (convert(shard[query], shard_no)
+                       for shard_no, shard in enumerate(self.shards))
+            return heapq.nlargest(self.num_best, itertools.chain(*results), key=lambda item: item[1])
+
+        # the trickiest combination: returning num_best results when query was a corpus
+        shard_results = []
+        for shard_no, shard in enumerate(self.shards):
+            shard_result = [convert(doc, shard_no) for doc in shard[query]]
+            shard_results.append(shard_result)
+        result = []
+        for parts in itertools.izip(*shard_results):
+            merged = heapq.nlargest(self.num_best, itertools.chain(*parts), key=lambda item: item[1])
+            result.append(merged)
         return result
 
 
@@ -351,8 +370,6 @@ class Similarity(interfaces.SimilarityABC):
             chunksize = self.chunksize
 
         for shard in self.shards:
-            # split each shard index into smaller chunks (of size self.chunksize) and
-            # use each chunk as a query
             query = shard.get_index().index
             for chunk_start in xrange(0, query.shape[0], chunksize):
                 # scipy.sparse doesn't allow slicing beyond real size of the matrix
@@ -378,6 +395,7 @@ class Similarity(interfaces.SimilarityABC):
 #endclass Similarity
 
 
+
 class MatrixSimilarity(interfaces.SimilarityABC):
     """
     Compute similarity against a corpus of documents by storing the index matrix
@@ -417,7 +435,17 @@ class MatrixSimilarity(interfaces.SimilarityABC):
             for docno, vector in enumerate(corpus):
                 if docno % 1000 == 0:
                     logger.debug("PROGRESS: at document #%i/%i" % (docno, len(corpus)))
-                self.index[docno] = matutils.unitvec(matutils.sparse2full(vector, num_features))
+                # individual documents in fact may be in numpy.scipy.sparse format as well.
+                # it's not documented because other it's not fully supported throughout.
+                # the user better know what he's doing (no normalization, must
+                # explicitly supply num_features etc).
+                if isinstance(vector, numpy.ndarray):
+                    pass
+                elif scipy.sparse.issparse(vector):
+                    vector = vector.toarray().flatten()
+                else:
+                    vector = matutils.unitvec(matutils.sparse2full(vector, num_features))
+                self.index[docno] = vector
 
 
     def __len__(self):
@@ -519,8 +547,10 @@ class SparseMatrixSimilarity(interfaces.SimilarityABC):
                 # no MmCorpus, use the slower version (or maybe user supplied the
                 # num_* params in constructor)
                 pass
-            self.index = matutils.corpus2csc((matutils.unitvec(vector) for vector in corpus),
-                                              num_terms=num_terms, num_docs=num_docs, num_nnz=num_nnz,
+            corpus = (matutils.scipy2sparse(v) if scipy.sparse.issparse(v) else
+                      (matutils.full2sparse(v) if isinstance(v, numpy.ndarray) else
+                       matutils.unitvec(v)) for v in corpus)
+            self.index = matutils.corpus2csc(corpus, num_terms=num_terms, num_docs=num_docs, num_nnz=num_nnz,
                                               dtype=dtype, printprogress=10000).T
 
             # convert to Compressed Sparse Row for efficient row slicing and multiplications
@@ -604,4 +634,3 @@ class SparseMatrixSimilarity(interfaces.SimilarityABC):
         result.index.data, result.index.indptr, result.index.indices = data, indptr, indices
         return result
 #endclass SparseMatrixSimilarity
-
